@@ -169,6 +169,7 @@ class AssertionResult:
         self._exception: Exception = None
         self.actual: dict = None
         self._expected: dict = None
+        self.is_builtin = False
 
     def status(self):
         return self._success
@@ -193,7 +194,7 @@ class AssertionResult:
         if self._exception:
             return self
 
-        if not self.actual or not self._expected:
+        if self.actual is None or not self._expected:
             return self.fail_with_assertion_implementation_error()
 
         return self
@@ -339,10 +340,20 @@ class AssertionEngine:
         for t in self.assertions_content:
             # only append specified table's assertions
             if t in selected_tables:
-                for ta in self.assertions_content[t].get('tests', []):
+                if self.assertions_content[t] is None:
+                    continue
+                table_assertions = self.assertions_content[t].get('tests') \
+                    if self.assertions_content[t].get('tests') else []
+                for ta in table_assertions:
                     self.assertions.append(AssertionContext(t, None, ta))
-                for c in self.assertions_content[t].get('columns', {}):
-                    for ca in self.assertions_content[t]['columns'][c].get('tests', []):
+
+                columns_content = self.assertions_content[t].get('columns') \
+                    if self.assertions_content[t].get('columns') else {}
+                for c in columns_content:
+                    if columns_content[c] is None:
+                        continue
+                    column_assertions = columns_content[c].get('tests') if columns_content[c].get('tests') else []
+                    for ca in column_assertions:
                         self.assertions.append(AssertionContext(t, c, ca))
 
     def load_all_assertions_for_validation(self) -> (List[str], List[str]):
@@ -351,10 +362,20 @@ class AssertionEngine:
         self.assertions = []
 
         for t in self.assertions_content:
-            for ta in self.assertions_content[t].get('tests', []):
+            if self.assertions_content[t] is None:
+                continue
+            table_assertions = self.assertions_content[t].get('tests') \
+                if self.assertions_content[t].get('tests') else []
+            for ta in table_assertions:
                 self.assertions.append(AssertionContext(t, None, ta))
-            for c in self.assertions_content[t].get('columns', {}):
-                for ca in self.assertions_content[t]['columns'][c].get('tests', []):
+
+            columns_content = self.assertions_content[t].get('columns') \
+                if self.assertions_content[t].get('columns') else {}
+            for c in columns_content:
+                if columns_content[c] is None:
+                    continue
+                column_assertions = columns_content[c].get('tests') if columns_content[c].get('tests') else []
+                for ca in column_assertions:
                     self.assertions.append(AssertionContext(t, c, ca))
 
         return passed_assertion_files, failed_assertion_files
@@ -501,8 +522,10 @@ class AssertionEngine:
         from piperider_cli.assertion_engine.types import get_assertion
         try:
             assertion_instance = get_assertion(assertion.name)
+
             try:
                 result = assertion_instance.execute(assertion, assertion.table, assertion.column, metrics_result)
+                assertion.is_builtin = assertion_instance.__class__.__module__.startswith(get_assertion.__module__)
                 result.validate()
             except Exception as e:
                 assertion.result.fail_with_exception(e)
@@ -548,6 +571,8 @@ class AssertionEngine:
         def to_dirs(path_list: str):
             if path_list is None:
                 return []
+            if sys.platform == 'win32':
+                return [x.strip() for x in path_list.split(';')]
             return [x.strip() for x in path_list.split(':')]
 
         plugin_dirs = []
