@@ -1,5 +1,4 @@
 import os.path
-import re
 import sys
 
 import click
@@ -7,37 +6,21 @@ import sentry_sdk
 from rich.console import Console
 from rich.syntax import Syntax
 
-from piperider_cli import __version__, event
+from piperider_cli import __version__, sentry_dns, sentry_env, event
 from piperider_cli.adapter import DbtAdapter
 from piperider_cli.assertion_generator import AssertionGenerator
+from piperider_cli.compare_report import CompareReport
 from piperider_cli.event.track import TrackCommand
+from piperider_cli.generate_report import GenerateReport
 from piperider_cli.guide import Guide
 from piperider_cli.initializer import Initializer
 from piperider_cli.runner import Runner
-from piperider_cli.generate_report import GenerateReport
-from piperider_cli.compare_report import CompareReport
 from piperider_cli.validator import Validator
 
-
-def set_sentry_env():
-    if '.dev' in __version__:
-        return 'development'
-    elif re.match('^\d+\.\d+\.\d+\.\d{8}[a|b|rc]?.*$', __version__):
-        return 'nightly'
-    elif 'a' in __version__:
-        return 'alpha'
-    elif 'b' in __version__:
-        return 'beta'
-    elif 'rc' in __version__:
-        return 'release-candidate'
-    return 'production'
-
-
-sentry_env = set_sentry_env()
 release_version = __version__ if sentry_env != 'development' else None
 
 sentry_sdk.init(
-    "https://41930bf397884adfb2617fe350231439@o1081482.ingest.sentry.io/6463955",
+    sentry_dns,
     environment=sentry_env,
     release=release_version,
     # Set traces_sample_rate to 1.0 to capture 100%
@@ -46,6 +29,7 @@ sentry_sdk.init(
     traces_sample_rate=1.0
 )
 sentry_sdk.set_tag("piperider.version", __version__)
+sentry_sdk.set_tag("platform", sys.platform)
 
 event.init()
 
@@ -73,10 +57,11 @@ def cli(ctx: click.Context):
         Guide().show_tips(ctx.command.name)
 
 
+# It works after click>=8
 cli.command_class = TrackCommand
 
 
-@cli.command(short_help='Show version information.')
+@cli.command(short_help='Show version information.', cls=TrackCommand)
 def version():
     'Show version information.'
     console = Console()
@@ -95,7 +80,7 @@ def version():
         pass
 
 
-@cli.command(short_help='Initialize a PipeRider project.')
+@cli.command(short_help='Initialize a PipeRider project.', cls=TrackCommand)
 @click.option('--no-auto-search', type=click.BOOL, default=False, is_flag=True,
               help="Disable auto detection of dbt projects.")
 @click.option('--dbt-project-dir', type=click.Path(exists=True), default=None,
@@ -139,7 +124,7 @@ def init(**kwargs):
         console.print(config)
 
 
-@cli.command(short_help='Check project configuration.')
+@cli.command(short_help='Check project configuration.', cls=TrackCommand)
 @add_options(debug_option)
 def diagnose(**kwargs):
     'Check project configuration, datasource, connections, and assertion configuration.'
@@ -153,10 +138,10 @@ def diagnose(**kwargs):
         sys.exit(1)
 
 
-@cli.command(short_help='Profile data source, run assertions, and generate report(s).')
+@cli.command(short_help='Profile data source, run assertions, and generate report(s).', cls=TrackCommand)
 @click.option('--datasource', default=None, type=click.STRING, help='Datasource to use.', metavar='DATASOURCE_NAME')
 @click.option('--table', default=None, type=click.STRING, help='Table to use.', metavar='TABLE_NAME')
-@click.option('--output', default=None, type=click.Path(), help='Directory to save the results.')
+@click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
 @click.option('--no-interaction', is_flag=True, help='Disable interactive mode.')
 @click.option('--skip-report', is_flag=True, help='Skip generating report.')
 @click.option('--skip-recommend', is_flag=True, help='Skip recommending assertions.')
@@ -182,10 +167,10 @@ def run(**kwargs):
                       skip_recommend=skip_recommend,
                       dbt_command=dbt_command)
     if not skip_report and ret == 0:
-        GenerateReport.exec()
+        GenerateReport.exec(None, output)
 
 
-@cli.command(short_help='Generate recommended assertions.')
+@cli.command(short_help='Generate recommended assertions.', cls=TrackCommand)
 @click.option('--input', default=None, type=click.Path(exists=True), help='Specify the raw result file.')
 @add_options(debug_option)
 def generate_assertions(**kwargs):
@@ -194,22 +179,29 @@ def generate_assertions(**kwargs):
     AssertionGenerator.exec(input=input)
 
 
-@cli.command(short_help='Generate a report.')
+@cli.command(short_help='Generate a report.', cls=TrackCommand)
 @click.option('--input', default=None, type=click.Path(exists=True), help='Specify the raw result file.')
+@click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
 @add_options(debug_option)
 def generate_report(**kwargs):
     'Generate a report from the latest raw result or specified result. By default, the raw results are saved in ".piperider/outputs".'
-
-    GenerateReport.exec(input=kwargs.get('input'))
+    GenerateReport.exec(input=kwargs.get('input'), output=kwargs.get('output'))
 
 
-@cli.command(short_help='Compare two existing reports.')
+@cli.command(short_help='Compare two existing reports.', cls=TrackCommand)
 @click.option('--base', default=None, type=click.Path(exists=True), help='Specify the base report file.')
-@click.option('--input', default=None, type=click.Path(exists=True), help='Specify the report file to be compared.')
+@click.option('--target', default=None, type=click.Path(exists=True), help='Specify the report file to be compared.')
+@click.option('--last', default=None, is_flag=True, help='Compare the last two reports.')
+@click.option('--datasource', default=None, type=click.STRING, metavar='DATASOURCE_NAME',
+              help='Specify the datasource.')
+@click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
 @add_options(debug_option)
 def compare_reports(**kwargs):
     'Compare two existing reports selected in interactive mode or by option.'
 
     a = kwargs.get('base')
-    b = kwargs.get('input')
-    CompareReport.exec(a=a, b=b)
+    b = kwargs.get('target')
+    last = kwargs.get('last')
+    datasource = kwargs.get('datasource')
+    CompareReport.exec(a=a, b=b, last=last, datasource=datasource, output=kwargs.get('output'),
+                       debug=kwargs.get('debug', False))
