@@ -11,7 +11,9 @@ from piperider_cli.error import \
     PipeRiderConfigError, \
     PipeRiderInvalidDataSourceError, \
     DbtProjectNotFoundError, \
-    DbtProfileNotFoundError
+    DbtProfileNotFoundError, \
+    DbtProjectInvalidError, \
+    DbtProfileInvalidError
 
 PIPERIDER_WORKSPACE_NAME = '.piperider'
 PIPERIDER_CONFIG_PATH = os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME, 'config.yml')
@@ -58,7 +60,10 @@ class Configuration(object):
             raise DbtProjectNotFoundError(dbt_project_path)
 
         with open(dbt_project_path, 'r') as fd:
-            dbt_project = yaml.safe_load(fd)
+            try:
+                dbt_project = yaml.safe_load(fd)
+            except Exception as e:
+                raise DbtProjectInvalidError(dbt_project_path, e)
 
         if not os.path.exists(os.path.expanduser(dbt_profile_path)):
             raise DbtProfileNotFoundError(dbt_profile_path)
@@ -81,6 +86,10 @@ class Configuration(object):
         if type_name not in DATASOURCE_PROVIDERS:
             raise PipeRiderInvalidDataSourceError(type_name, dbt_profile_path)
 
+        # Set 'pass' as the alias of 'password'
+        if credential.get('pass') and credential.get('password') is None:
+            credential['password'] = credential.pop('pass')
+
         datasource_class = DATASOURCE_PROVIDERS[type_name]
         datasource: DataSource = datasource_class(name=profile_name, dbt=dbt, credential=credential)
         datasource.show_installation_information()
@@ -119,6 +128,9 @@ class Configuration(object):
                     profile_path = os.path.expanduser(profile_path)
                 profile = _load_dbt_profile(profile_path)
                 credential = profile.get(dbt.get('profile'), {}).get('outputs', {}).get(dbt.get('target', {}))
+                # TODO: extract duplicate code from func 'from_dbt_project'
+                if credential.get('pass') and credential.get('password') is None:
+                    credential['password'] = credential.pop('pass')
                 data_source = datasource_class(name=ds.get('name'), dbt=dbt, credential=credential)
             else:
                 try:
@@ -211,4 +223,7 @@ def _load_dbt_profile(path):
     env.filters['as_number'] = as_number
     env.filters['as_text'] = as_text
     template = env.get_template(os.path.basename(path))
-    return yaml.safe_load(template.render())
+    try:
+        return yaml.safe_load(template.render())
+    except Exception as e:
+        raise DbtProfileInvalidError(path, e)
