@@ -1,18 +1,36 @@
-import os
 import json
+import os
+import platform
 import sys
-from re import I
 import time
-import requests
-import portalocker
 from datetime import datetime
+
+import portalocker
+import requests
+
 from piperider_cli import __version__
-from piperider_cli.workspace import PIPERIDER_WORKSPACE_NAME
+from piperider_cli.configuration import PIPERIDER_WORKSPACE_NAME
 
 PIPERIDER_WORKING_DIR = os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME)
 PIPERIDER_EVENT_PATH = os.path.join(PIPERIDER_WORKING_DIR, '.unsend_events.json')
 
 
+def is_executed_manually():
+    # some CI service puts the "CI" var
+    if os.environ.get('CI', 'false') == 'true':
+        return False
+
+    # For CiecleCI exceptions
+    if os.environ.get('CIRCLECI', 'false') == 'true':
+        return False
+
+    # if not tty, it probably runs automatically
+    if not sys.stdout.isatty():
+        return False
+
+    return True
+
+
 class Collector:
     def __init__(self):
         self._api_endpoint = 'https://api.amplitude.com/2/httpapi'
@@ -20,6 +38,7 @@ class Collector:
         self._user_id = None
 
         self._unsend_events_file = PIPERIDER_EVENT_PATH
+        self._delete_threshold = 1000
         self._upload_threshold = 10
 
         self._check_required_files()
@@ -47,14 +66,19 @@ class Collector:
             user_properties=dict(
                 version=__version__,
                 python_version=python_version,
+                manual=is_executed_manually()
             ),
             event_properties=prop,
+            platform=sys.platform,
+            os_version=platform.platform(),
+            app_version=__version__,
         )
 
         # TODO: handle exception when writing to file
         self._store_to_file(event)
         if self._is_full():
             self.send_events()
+        self._cleanup_unsend_events()
 
     def _check_required_files(self):
         if not os.path.exists(PIPERIDER_WORKING_DIR):
@@ -100,3 +124,17 @@ class Collector:
             f.seek(0)
             f.truncate()
             f.write(json.dumps(o))
+
+    def _cleanup_unsend_events(self):
+        with portalocker.Lock(self._unsend_events_file, 'r+', timeout=5) as f:
+            o = json.loads(f.read())
+            events = o.get('unsend_events', None)
+            if events is None:
+                o['unsend_events'] = []
+
+            while len(o['unsend_events']) > self._delete_threshold:
+                o['unsend_events'].pop(0)
+
+            f.seek(0)
+            f.truncate()
+            f.write(json.dumps(o))
