@@ -1,16 +1,30 @@
 import json
 import os
 import re
+import shutil
 
 from rich.console import Console
 
-from piperider_cli import clone_directory
+from piperider_cli import __version__, sentry_dns, sentry_env, event
+from piperider_cli import clone_directory, raise_exception_when_output_directory_not_writable
 from piperider_cli.configuration import PIPERIDER_WORKSPACE_NAME
 from piperider_cli.error import PipeRiderNoProfilingResultError
 
 PIPERIDER_OUTPUT_PATH = os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME, 'outputs')
 
 
+def prepare_piperider_metadata():
+    metadata = {
+        'name': 'PipeRider',
+        'sentry_env': sentry_env,
+        'sentry_dns': sentry_dns,
+        'version': __version__,
+        'amplitude_api_key': event._get_api_key(),
+        'amplitude_user_id': event._collector._user_id,
+    }
+    return metadata
+
+
 def _validate_input_result(result):
     for f in ['tables', 'id', 'created_at', 'datasource']:
         if f not in result:
@@ -23,10 +37,17 @@ def setup_report_variables(template_html: str, is_single: bool, data):
         output = json.dumps(data)
     else:
         output = data
+    metadata = json.dumps(prepare_piperider_metadata())
     if is_single:
-        variables = f'<script id="piperider-report-variables">\nwindow.PIPERIDER_SINGLE_REPORT_DATA={output};window.PIPERIDER_COMPARISON_REPORT_DATA="";</script>'
+        variables = f'<script id="piperider-report-variables">\n' \
+                    f'window.PIPERIDER_METADATA={metadata};' \
+                    f'window.PIPERIDER_SINGLE_REPORT_DATA={output};' \
+                    f'window.PIPERIDER_COMPARISON_REPORT_DATA="";</script>'
     else:
-        variables = f'<script id="piperider-report-variables">\nwindow.PIPERIDER_SINGLE_REPORT_DATA="";window.PIPERIDER_COMPARISON_REPORT_DATA={output};</script>'
+        variables = f'<script id="piperider-report-variables">\n' \
+                    f'window.PIPERIDER_METADATA={metadata};' \
+                    f'window.PIPERIDER_SINGLE_REPORT_DATA="";' \
+                    f'window.PIPERIDER_COMPARISON_REPORT_DATA={output};</script>'
     html_parts = re.sub(r'<script id="piperider-report-variables">.+?</script>', '#PLACEHOLDER#', template_html).split(
         '#PLACEHOLDER#')
     html = html_parts[0] + variables + html_parts[1]
@@ -59,8 +80,9 @@ def _get_run_json_path(input=None):
 
 class GenerateReport():
     @staticmethod
-    def exec(input=None):
+    def exec(input=None, output=None):
         console = Console()
+        raise_exception_when_output_directory_not_writable(output)
 
         from piperider_cli import data
         report_template_dir = os.path.join(os.path.dirname(data.__file__), 'report', 'single-report')
@@ -79,8 +101,17 @@ class GenerateReport():
 
         console.print(f'[bold dark_orange]Generating reports from:[/bold dark_orange] {run_json_path}')
 
-        dir = os.path.dirname(run_json_path)
-        clone_directory(report_template_dir, dir)
+        def output_report(target_directory):
+            clone_directory(report_template_dir, target_directory)
+            _generate_static_html(result, report_template_html, target_directory)
 
-        _generate_static_html(result, report_template_html, dir)
-        console.print(f"Report generated in {dir}/index.html")
+        # output the report to the default directory (same with the run.json)
+        default_output_directory = os.path.dirname(run_json_path)
+        output_report(default_output_directory)
+
+        if output:
+            output_report(output)
+            shutil.copyfile(run_json_path, os.path.join(output, os.path.basename(run_json_path)))
+            console.print(f"Report generated in {output}/index.html")
+        else:
+            console.print(f"Report generated in {default_output_directory}/index.html")
