@@ -13,7 +13,9 @@ from rich.progress import Progress, Column, TextColumn, BarColumn, TimeElapsedCo
 from rich.table import Table
 from sqlalchemy import create_engine, inspect
 
-from piperider_cli import convert_to_tzlocal, datetime_to_str
+from piperider_cli import convert_to_tzlocal, datetime_to_str, clone_directory, \
+    raise_exception_when_output_directory_not_writable
+from piperider_cli import event
 from piperider_cli.adapter import DbtAdapter
 from piperider_cli.assertion_engine import AssertionEngine
 from piperider_cli.assertion_engine.recommender import RECOMMENDED_ASSERTION_TAG
@@ -23,6 +25,26 @@ from piperider_cli.error import PipeRiderCredentialError
 from piperider_cli.profiler import Profiler, ProfilerEventHandler
 
 
+class RunEventPayload:
+
+    def __init__(self):
+        # all fields
+        self.tables = 0
+        self.columns = []
+        self.rows = []
+        self.dbt_command = ''
+        self.passed_assertions = 0
+        self.failed_assertions = 0
+        self.passed_dbt_testcases = 0
+        self.failed_dbt_testcases = 0
+        self.build_in_assertions = 0
+        self.custom_assertions = 0
+        self.recommended_assertions = 0
+
+    def to_dict(self):
+        return self.__dict__
+
+
 class RichProfilerEventHandler(ProfilerEventHandler):
 
     def __init__(self, tables, ds):
@@ -259,23 +281,21 @@ def _show_dbt_test_result_summary(dbt_test_results, table: str = None):
     ascii_table.add_column('#DBT Tests Executed', style='bold blue', justify='right')
     ascii_table.add_column('#DBT Tests Failed', style='bold red', justify='right')
 
-    num_of_testcases = 0
-    num_of_passed_testcases = 0
-
     for k, v in dbt_test_results.items():
         if table is None or k == table:
-            for column, results in v['columns'].items():
-                for r in results:
-                    num_of_testcases += 1
-                    if r.get('status') == 'passed':
-                        num_of_passed_testcases += 1
+            num_of_testcases = 0
+            num_of_failed_testcases = 0
+
+            for results in v['columns'].values():
+                num_of_testcases += len(results)
+                num_of_failed_testcases += sum([1 for r in results if r.get('status') != 'passed'])
 
-            num_of_failed_testcases = num_of_testcases - num_of_passed_testcases
             ascii_table.add_row(
                 k,
                 Pretty(num_of_testcases),
                 Pretty(num_of_failed_testcases),
             )
+
     return ascii_table
 
 
@@ -312,20 +332,14 @@ def _show_summary(profiled_result, assertion_results, assertion_exceptions, dbt_
 def _show_table_summary(ascii_table: Table, table: str, profiled_result, assertion_results):
     profiled_columns = profiled_result['tables'][table].get('col_count')
     num_of_testcases = 0
-    num_of_passed_testcases = 0
     num_of_failed_testcases = 0
-    failed_testcases = []
 
     if assertion_results:
         for r in assertion_results:
             if r.table == table:
                 num_of_testcases += 1
-                if r.result.status():
-                    num_of_passed_testcases += 1
-                else:
-                    failed_testcases.append(r)
-
-    num_of_failed_testcases = num_of_testcases - num_of_passed_testcases
+                if not r.result.status():
+                    num_of_failed_testcases += 1
 
     ascii_table.add_row(
         table,
@@ -333,9 +347,6 @@ def _show_table_summary(ascii_table: Table, table: str, profiled_result, asserti
         Pretty(num_of_testcases),
         Pretty(num_of_failed_testcases),
     )
-    # console.print(f'  {profiled_columns} columns profiled')
-
-    pass
 
 
 def _transform_assertion_result(table: str, results):
@@ -400,12 +411,11 @@ def _get_table_list(table, default_schema, dbt_adapter):
     return tables
 
 
-def prepare_output_path(created_at, ds, output):
+def prepare_default_output_path(created_at, ds):
     latest_symlink_path = os.path.join(PIPERIDER_OUTPUT_PATH, 'latest')
     output_path = os.path.join(PIPERIDER_OUTPUT_PATH,
                                f"{ds.name}-{convert_to_tzlocal(created_at).strftime('%Y%m%d%H%M%S')}")
-    if output:
-        output_path = output
+
     if not os.path.exists(output_path):
         os.makedirs(output_path, exist_ok=True)
 
@@ -428,17 +438,30 @@ def _append_descriptions(profile_result):
             column_v['description'] = 'Description: N/A'
 
 
+def _clean_up_null_properties(table_results):
+    removed = []
+    for col_name, props in table_results.get('columns', {}).items():
+        for k, v in props.items():
+            if v is None:
+                removed.append(dict(col=col_name, key=k))
+
+    for r in removed:
+        del table_results['columns'][r['col']][r['key']]
+
+
 def _append_descriptions_from_assertion(profile_result):
     engine = AssertionEngine(None)
     engine.load_assertion_content()
     for table_name, table_v in engine.assertions_content.items():
-        if table_name not in profile_result['tables']:
+        if table_name not in profile_result['tables'] or table_v is None:
             continue
         table_desc = table_v.get('description', '')
         if table_desc:
             profile_result['tables'][table_name]['description'] = f'{table_desc} - via PipeRider'
-        for column_name, column_v in table_v.get('columns', {}).items():
-            if column_name not in profile_result['tables'][table_name]['columns']:
+
+        columns_content = table_v.get('columns') if table_v.get('columns') else {}
+        for column_name, column_v in columns_content.items():
+            if column_name not in profile_result['tables'][table_name]['columns'] or column_v is None:
                 continue
             column_desc = column_v.get('description', column_name)
             if column_desc:
@@ -446,13 +469,53 @@ def _append_descriptions_from_assertion(profile_result):
                     'description'] = f'{column_desc} - via PipeRider'
 
 
+def _analyse_and_log_run_event(profiled_result, assertion_results, dbt_test_results, dbt_command):
+    tables = profiled_result.get('tables', [])
+    event_payload = RunEventPayload()
+    event_payload.tables = len(tables)
+
+    # Table info
+    for t in tables:
+        event_payload.columns.append(profiled_result['tables'][t]['col_count'])
+        event_payload.rows.append(profiled_result['tables'][t]['row_count'])
+
+    # Count PipeRider assertions
+    for r in assertion_results or []:
+        if r.is_builtin:
+            event_payload.build_in_assertions += 1
+        else:
+            event_payload.custom_assertions += 1
+
+        if RECOMMENDED_ASSERTION_TAG in r.tags:
+            event_payload.recommended_assertions += 1
+
+        if r.result.status():
+            event_payload.passed_assertions += 1
+        else:
+            event_payload.failed_assertions += 1
+
+    # Count dbt-test cases
+    if dbt_test_results:
+        for table_name, table in dbt_test_results.items():
+            for results in table['columns'].values():
+                for r in results:
+                    if r.get('status') == 'passed':
+                        event_payload.passed_dbt_testcases += 1
+                    else:
+                        event_payload.failed_dbt_testcases += 1
+
+    event_payload.dbt_command = dbt_command
+    event.log_event(event_payload.to_dict(), 'run')
+
+
 class Runner():
     @staticmethod
     def exec(datasource=None, table=None, output=None, interaction=True, skip_report=False, dbt_command='',
              skip_recommend=False):
         console = Console()
-        configuration = Configuration.load()
+        raise_exception_when_output_directory_not_writable(output)
 
+        configuration = Configuration.load()
         datasources = {}
         datasource_names = []
         for ds in configuration.dataSources:
@@ -519,7 +582,7 @@ class Runner():
         except Exception as e:
             raise Exception(f'Profiler Exception: {type(e).__name__}(\'{e}\')')
 
-        output_path = prepare_output_path(created_at, ds, output)
+        output_path = prepare_default_output_path(created_at, ds)
 
         # output profiling result
         with open(os.path.join(output_path, ".profiler.json"), "w") as f:
@@ -555,6 +618,7 @@ class Runner():
         for t in profile_result['tables']:
             profile_result['tables'][t]['piperider_assertion_result'] = _transform_assertion_result(t,
                                                                                                     assertion_results)
+            _clean_up_null_properties(profile_result['tables'][t])
         _show_summary(profile_result, assertion_results, assertion_exceptions, dbt_test_results)
         _show_recommended_assertion_notice_message(console, assertion_results)
 
@@ -565,6 +629,13 @@ class Runner():
 
         with open(output_file, 'w') as f:
             f.write(json.dumps(profile_result, indent=4))
+
+        if output:
+            clone_directory(output_path, output)
+
         if skip_report:
-            console.print(f'Results saved to {output_path}')
+            console.print(f'Results saved to {output if output else output_path}')
+
+        _analyse_and_log_run_event(profile_result, assertion_results, dbt_test_results, dbt_command)
+
         return 0
